/* ---------------------------------------------------
   Program konwertuje znaki ze strony kodowej win-1250
   (w pliku wejsciowym) na iso 8859-2 (plik wynikowy)

   autor: Piotr Grabowski garbi@figaro.ae.katowice.pl
   --------------------------------------------------- */

// -------------- Pliki naglowkowe --------------
#include <stdlib.h> // Bibliteka standardowa
                    // system, EXIT_FAILURE, EXIT_SUCCESS

#include <iostream> // Strumienie (input output stream)
                    // cout, EOF

#include <fstream>  // Operacje na plikach (file stream)
                    // ifstream, ofstream
// ----------------------------------------------

// -------------- Deklaracje funkcji -------------
void Uzycie(char *NazwaProgramu);
int KonwertujPlik(char *PlikWin, char *PlikIso);
// -----------------------------------------------

// Przekazywanie argumentow do programu:
//  int argc     - liczba przekazywanych argumentow + 1 
//  char *argv[] - tablica wskaznikw, ktore sa typu znakowego;
//                 zawiera ona aktualnie przekazane argumenty
int main(int argc, char *argv[])
{
	if (argc != 3) // nazwa_programu plik_win plik_iso
	{
		// Jesli nie przekazemy wlasciwej liczby argumentow do programu
		Uzycie(argv[0]);     // argv[0] - nazwa uruchomionego programu
		return EXIT_FAILURE; // kod bledu zdef. w stdlib.h
	}
	else
	{
		// Wywolaj funkcje konwertujaca
		KonwertujPlik(argv[1], argv[2]);
	}
	system("pause");
	return EXIT_SUCCESS;
}

void Uzycie(char *NazwaProgramu)
{
	cout << " Program konwertuje plik ze strony kodowej \n"
		" Windows-1250 na strone ISO 8859-2             \n"
		" z wykorzystaniem bibliotek standardowych C++  \n"
		" autor: Piotr Grabowski garbi@figaro.ae.katowice.pl \n\n"
		" Uzycie: " << NazwaProgramu << " win1250 iso8859-2  \n\n"
		" gdzie:  " << NazwaProgramu << " - nazwa programu konwertujacego \n"
		"       win1250   - plik majacy znaki zakodowane w Windows-1250 \n"
		"       iso8859-2 - plik wynikowy (tablica znakow ISO 8859-2) \n\n";
}

int KonwertujPlik(char *PlikWin, char *PlikIso)
{
ifstream win(PlikWin); // Plik wejsciowy: obiekt win
                       // klasy i(nput)f(ile)stream
					   // z pliku fstream.h

	if (!win) // if (win == NULL)
	{
		cout << " Nie moge otworzyc takiego pliku! \n";
		return EXIT_FAILURE;
	}
	else
	{
	unsigned char znak;    // Wczytuj znak z pliku jako typ znakowy bez znaku!!!
	ofstream iso(PlikIso); // Plik wynikowy: obiekt iso
                           // klasy o(utput)f(ile)stream
                           // z pliku fstream.h
		while (win.get(znak) != NULL) // NULL w <stddef.h>
                                      // wlaczanym przez <iostream>
		// Symfonia C++, str. 633-634 (wyd. IV/tom III)
		{
			switch (znak)
			{
				// Konwertuj wybrane znaki
				// (wart. dziesiatkowe)
				//   win    ->    ISO
				case 140 : znak = 166;
					break;
				case 143 : znak = 172;
					break;
				case 156 : znak = 182;
					break;
				case 159 : znak = 188;
					break;
				case 165 : znak = 161;
					break;
				case 185 : znak = 187;
					break;
			} // koniec switch
			if (!iso.put(znak))  // Kazdy znak do pliku_iso +
			                     // sprawdzenie rezultatu
			{
				cout << " Blad zapisu do pliku! \n";
				break; // Wyjdz z petli while
			} 
		} // koniec while
		cout << " Konwersja zakonczona ";
		if (!win.eof()) // Jedyne poprawne zakonczenie konwersji to
						// osiagniecie konca konwertowanego pliku -
						// ustawienie flagi ios::eofbit
		{
			cout << "niepowodzeniem \n";
			return EXIT_FAILURE;
		}
		else
		{
			cout << "pomyslnie \n";
			return EXIT_SUCCESS;
		}
	} // koniec else
}
