// KONWERTER POLSKICH ZNAKOW v0.5d

// autor: Piotr Grabowski
// garbi@figaro.ae.katowice.pl
// 
// ANSI C++

// -------- Pliki naglowkowe -------------
#include <iostream.h>  // strumienie std.: cout, get, put
#include <fstream.h>   // oper. na plikach: (i/o)fstream 
#include <stdlib.h>    // bibl. std.: EXIT_..., system
#include <ctype.h>     // tolower
// ---------------------------------------

// Deklaracja typu wyliczeniowego
enum kod {L, M, W, I, B, BLAD};
// L - IBM Latin-2 (cp852)
// M - Mazovia
// W - Windows EE (1250)
// I - ISO 8859-2 (ISO Latin 2)
// B - Bez polskich znakow
// BLAD - Zly parametr

// ----------- Funkcje globalne -------------
void uzycie(char *nazwa_programu);
// ------------------------------------------

// Deklaracja klasy, z obiektem static
// Sprawdzanie argumentow wewnatrz klasy
class Konwerter
{
private:
	// kodowanie plikow:
	kod _kod_we;        // wejsciowego
	kod _kod_wy;        // wynikowego
	// nazwy plikow:
	ifstream _zrodlo;   // wejsciowego
	ofstream _cel;      // wynikowego
	
	/* Dana, ktora jest okreslona jako statyczna (static)
	ma te wazna ceche, ze jest w pamieci tworzona jednokrotnie
	i jest wspolna dla wszystkich obiektow danej klasy.
	Co wiecej: istnieje nawet wtedy, gdy jeszcze nie
	zdefiniowalismy ani jednego egzemplarza obiektu tej klasy [S] */
	// czy ststic mozna zmieniac?
	// const na pewno nie.
	// Nie ma inicjalizacji obiektu const !!!????
	// bo tu nie wolno
	// Inicjalizacja dla konstr. obiektu, ale
	// static, wiec dla calej klasy ???
	static const unsigned char znaki[5][18];

public:
	Konwerter(char *, char, char *, char); // Deklar. konstrukt.
    ~Konwerter();          // Destr.
    kod zamien_na_enum(char literka);
    int sprawdz_arg	(void);
	int konwertuj_plik(void); // Dekl. metody (fun. skladowa)
};

// Definicja obiektu private static const,
// inicjalizacja tablicy poza zakresem waznosci!
// tak wolno z obiektem static
const unsigned char Konwerter::znaki[5][18] =
{
    {'', '', '', '', '', '', '', '', '',
	'', '', '', '', '', '', '', '', ''}, // IBM [L]atin 2 (cp852)
    {'', '', '', '', '', '', '', '', '',
	'', '', '', '', '', '', '', '', ''}, // [M]azovia
	{'', '', '', '', '', '', '', '', '',
	'', '', '', '', '', '', '', '', ''}, // [W]indows EE (1250)
	{'', '', '', '', '', '', '', '', '',
	'', '', '', '', '', '', '', '', ''}, // [I]SO Latin 2 (8859-2)
	{'A', 'C', 'E', 'L', 'N', 'O', 'S', 'Z', 'Z',
	'a', 'c', 'e', 'l', 'n', 'o', 's', 'z', 'z'}  // [B]ez pol. znakow
};

// Konstruktor klasy Konwerter
Konwerter::Konwerter(char *plik_we, char kod_we, char *plik_wy, char kod_wy)
{
    // Dokonaj inicjalizacji skladowych prywatnych
    _kod_we = zamien_na_enum(kod_we); // tylko pierwsza literke
    _kod_wy = zamien_na_enum(kod_wy); //  -------- || --------
    // Otworz strumienie plikowe
    _zrodlo.open(plik_we);
    _cel.open(plik_wy);
}

// Destruktor
Konwerter::~Konwerter()
{
    // Zamknij strumienie plikowe
    _zrodlo.close();
    _cel.close();
}

// Zamienia argumenty stron kodowych
// ze znaku na typ wyliczeniowy kod
kod Konwerter::zamien_na_enum(char literka)
{
	literka = tolower(literka);
	
	if (literka == 'l') return L;
	else
	if (literka == 'm') return M;
	else
	if (literka == 'w') return W;
	else
	if (literka == 'i') return I;
	else
	if (literka == 'b') return B;
	else
	return BLAD;
}

// Funkcja sprawdza przekazane do programu argumenty
int Konwerter::sprawdz_arg()
{
	if (_kod_we == BLAD || _kod_we == B ||
		_kod_wy == BLAD || _kod_we == _kod_wy)
	{
		cout << " Podane argumenty sa niepoprawne! \n";
		return EXIT_FAILURE;
	}

	if (!_zrodlo) // if (_zrodlo.fail())
	{
		cout << " Nie moge otworzyc pliku zrodlowego! \n";
		return EXIT_FAILURE;
	}
	
	if (!_cel)
	{
        cout << " Nie moge otworzyc pliku docelowego! \n";
		return EXIT_FAILURE;
	}

/*    if (_zrodlo == _cel)
    {
        cout << " Blad: Podane pliki sa identyczne! \n";
        return EXIT_FAILURE;
    }
*/	
	return EXIT_SUCCESS;
}

// Definicja:
// Funkcja konwertuje polskie znaki
int Konwerter::konwertuj_plik(void)
{
unsigned char znak;

	while (_zrodlo.get(znak) != NULL) // NULL w <stddef.h>
                                      // wlaczanym przez <iostream>
    // Symfonia C++, str. 633-634 (wyd. IV/tom III)
	{
		// Konwertuj wybrane znaki
		for (int i = 0; i < 18; i++)
		{
			if (znak == znaki[_kod_we][i])
				znak = znaki[_kod_wy][i];
		} // koniec for
		if (!_cel.put(znak))  // Kazdy znak do pliku wynikowego +
		                      // sprawdzenie rezultatu
		{
			cout << " Blad zapisu do pliku! \n";
			break; // Wyjdz z petli while
		} 
	} // koniec while
	cout << " Konwersja zakonczona ";
	if (!_zrodlo.eof()) // Jedyne poprawne zakonczenie konwersji to
	// osiagniecie konca konwertowanego pliku -
	// ustawienie flagi ios::eofbit
	{
		cout << "niepowodzeniem \n";
		return EXIT_FAILURE;
	}
	else
	{
		cout << "pomyslnie \n";
		return EXIT_SUCCESS;
	}
}

// Funkcja glowna
int main(int argc, char *argv[])
{
    cout << "\n KONWERTER POLSKICH ZNAKOW v0.5d \n\n";

	if (argc != 5) // nazwa_programu zrodlo kod_we cel kod_wy
	{
		// Jesli nie przekazemy wlasciwej liczby argumentow do programu
		uzycie(argv[0]);     // argv[0] - nazwa uruchomionego programu
	}
	else
	{
   		// Deklaracja obiektu klasy Konwerter
		Konwerter plik(argv[1], argv[2][0], argv[3], argv[4][0]);
		// Jesli argumenty s poprawne
        // wykonaj konwersje na rzecz tego obiektu
        if (plik.sprawdz_arg() == EXIT_SUCCESS)
    		plik.konwertuj_plik();
	}
    cout << endl;
	system("pause");
	return EXIT_SUCCESS;
}

// Pomoc
void uzycie(char *nazwa_programu)
{
	cout << "  Program konwertuje polskie znaki"
		" pomiedzy roznymi stronami kodowymi \n"
		"  autor: Piotr Grabowski garbi@figaro.ae.katowice.pl \n\n"
		" Uzycie: " << nazwa_programu << 
		"  [zrodlo] [kod zrodla] [cel] [kod celu] \n"
		" gdzie: \n"
		"       " << nazwa_programu << " - nazwa programu konwertujacego \n"
		"       zrodlo     - nazwa pliku wejsciowego \n"
		"       kod zrodla - strona kodowa pliku wejsciowego \n"
		"       cel        - nazwa pliku wynikowego \n"
		"       kod celu   - strona kodowa pliku wynikowego \n"
		" kod zrodla/celu: \n"
		"                  L - IBM Latin-2 (cp852) \n"
		"                  M - Mazovia \n"
		"                  W - Windows EE (1250) \n"
        "                  I - ISO 8859-2 (ISO Latin 2) \n"
		"                  B - Bez polskich znakow (tylko jako kod celu)\n"
        " UWAGI: 1) Nie konwertuj pliku z kodem zrodlowym programu \n"
        "        2) Poprawnosc wyniku zalezy od prawdziwosci zadeklarowanego \n"
        "           kodowania pliku zrodlowego \n"
        "        3) Jesli plik wynikowy istnieje to zostanie zastapiony \n";
}
// ------------ KONIEC PLIKU ------------------
